==> navigation_plus-1.0.1.tar.gz: navigation_plus/src/NavigationPlusUi.php <==
<?php

declare(strict_types=1);

namespace Drupal\navigation_plus;

use Drupal\Core\Render\Element;
use Drupal\Core\Routing\AdminContext;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Routing\RouteMatchInterface;
use Drupal\Core\Session\AccountProxyInterface;
use Drupal\Core\Extension\ExtensionPathResolver;
use Drupal\layout_builder\SectionStorageInterface;
use Symfony\Component\HttpFoundation\RequestStack;
use Drupal\Core\Entity\EntityDisplayRepositoryInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Symfony\Contracts\EventDispatcher\EventDispatcherInterface;

/**
 * Navigation + UI.
 */
final class NavigationPlusUi {

  use StringTranslationTrait;

  /**
   * @var \Drupal\Core\Entity\EntityInterface|NULL
   *   The entity derived from the route.
   *   @see deriveEntityFromRoute.
   */
  private ?EntityInterface $entity = NULL;

  public function __construct(
    private readonly RequestStack $requestStack,
    private readonly RouteMatchInterface $routeMatch,
    private readonly AdminContext $routerAdminContext,
    private readonly ModePluginManager $modeManager,
    private readonly AccountProxyInterface $currentUser,
    private readonly EventDispatcherInterface  $eventDispatcher,
    private readonly ExtensionPathResolver $extensionPathResolver,
    private readonly EntityDisplayRepositoryInterface $entityDisplayRepository,
  ) {}

  /**
   * Add toolbars.
   *
   * Add mode toolbars to the navigation module's left sidebar.
   *
   * @param array $variables
   *   The navigation sidebar render array.
   *
   * @throws \Drupal\Component\Plugin\Exception\PluginException
   */
  public function buildToolbars(&$variables) {
    $mode_state = $this->getMode();
    // Hide the standard navigation items if any modes are enabled.
    if ($mode_state !== 'none') {
      foreach ($variables['content']['content'] as &$navigation_item) {
        $navigation_item['#attributes']['class'][] = 'navigation-plus-hidden';
      }
    }
    foreach ($this->modeManager->getModePlugins() as $mode_id => $mode) {
      $variables['#cache']['contexts'][] = 'user.permissions';
      $variables['#cache']['contexts'][] = "cookies:navigationMode";

      // Add a button to toggle into the mode.
      $variables['#attached']['library'][] = 'navigation_plus/navigation_plus';
      $children = Element::children($variables['content']['footer']);
      $last_key = end($children);
      $variables['content']['footer'][$last_key]['content']["navigation_plus_toggle_{$mode_id}_mode"] = [
        '#type' => 'inline_template',
        '#template' => "<style>#toggle-{{mode_id}}-mode::before { mask-image: url({{icon_path}}); }</style><a id='toggle-{{mode_id}}-mode' data-mode='{{mode_id}}' data-drupal-tooltip='{{mode}}' data-drupal-tooltip-class='admin-toolbar__tooltip' class='navigation-plus-mode-button toolbar-button toolbar-button--collapsible{{toolbar_state}}' data-index-text='0' data-icon-text='{{icon_text}}' href='javascript:void(0)'><span class='toolbar-button__label'>{{label}}</span></a>",
        '#context' => [
          'mode_id' => $mode_id,
          'label' => $mode->label(),
          'icon_text' => $mode->label(),
          'toolbar_state' => $mode_state === $mode_id ? ' active' : '',
          'mode' => t('@label mode', ['@label' => $mode->label()]),
          'icon_path' => $mode->getIconPath(),
        ],
      ];

      // Add a toolbar for this mode.
      $toolbar_classes = ['toolbar-block'];
      if ($mode_state !== $mode_id) {
        $toolbar_classes[] = 'navigation-plus-hidden';
      }
      $variables['content']['content']["navigation_plus_$mode_id"] = [
        '#type' => 'container',
        '#attributes' => [
          'id' => "navigation-plus-$mode_id",
          'class' => $toolbar_classes,
        ],
        // Match the navigation module markup.
        'admin_toolbar_item' => [
          '#type' => 'container',
          '#attributes' => [
            'class' => ['admin-toolbar__item'],
          ],
          'title' => [
            '#type' => 'html_tag',
            '#tag' => 'h4',
            '#value' => t('Editing Toolbar'),
            '#attributes' => [
              'class' => ['visually-hidden', 'focusable'],
            ],
          ],
          'tools' => [
            '#theme' => 'item_list',
            '#attributes' => [
              'class' => ['toolbar-block__list'],
            ],
            '#items' => $mode->buildToolbarItems($variables, $mode),
          ],
        ],
        '#weight' => 99,
      ];
    }
  }

  /**
   * Build page top.
   *
   * Let modes add top and side bars.
   *
   * @param array $page_top
   *   The page_top render array.
   *
   * @return void
   *
   * @throws \Drupal\Component\Plugin\Exception\PluginException
   */
  public function buildPageTop(array &$page_top) {
    foreach ($this->modeManager->getModePlugins() as $mode_id => $mode) {
      $mode->buildBars($page_top, $mode);
    }
  }

  /**
   * Preprocess top bar.
   */
  public function preprocessTopBar(&$variables) {
    $variables['#cache']['contexts'][] = 'cookies:navigationMode';
    // Hide the navigation top bar when in a mode.
    if ($this->getMode() !== 'none') {
      $variables['attributes']['class'][] = 'navigation-plus-hidden';
    }
  }


  /**
   * Get mode.
   *
   * Many UI state items are stored in JS's Local and Session storage. Edit mode
   * is stored as a cookie so that the server can conditionally render the page
   * elements with attributes used for the editing UI. Rendering this server side
   * prevents a flashing that would occur if we waited till the JS was loaded
   * to enable the editing UI.
   *
   * @param string $cookie_name
   *   The name of the cookie.
   *
   * @return string
   *   Whether edit mode is enabled (Whether the toolbar is open or closed).
   */
  public function getMode(string $cookie_name = 'navigationMode'): string {
    // Cookies are for pages like /node/10
    return $this->requestStack->getCurrentRequest()->cookies->get($cookie_name) ??
      // Query parameters are for js requests like /lb-plus/place-block/overrides/node.16
      $this->requestStack->getCurrentRequest()->get($cookie_name) ??
      'none';
  }

  /**
   * Is valid view mode.
   *
   * @return bool
   *   Whether the view_mode is a valid view mode.
   */
  public function isValidViewMode(EntityInterface $entity, string $view_mode): bool {
    $valid_view_modes = $this->entityDisplayRepository->getViewModes($entity->getEntityTypeId());
    $valid_view_modes['default'] = TRUE;
    return !empty($valid_view_modes[$view_mode]);
  }

  /**
   * Derive entity from route.
   *
   * @return \Drupal\Core\Entity\EntityInterface|null
   *   Returns the entity for this route or NULL if there isn't one or there is
   *   more than one.
   */
  public function deriveEntityFromRoute() {
    if (empty($this->entity)) {
      $parameters = $this->routeMatch->getParameters()->all();
      $top_level_entity = NULL;
      foreach ($parameters as $parameter) {
        $entity = NULL;
        if ($parameter instanceof EntityInterface) {
          $entity = $parameter;
        }
        if ($parameter instanceof SectionStorageInterface) {
          $entity = $parameter->getContextValue('entity');
        }
        if (!empty($entity) && is_null($top_level_entity)) {
          $top_level_entity = $entity;
        } elseif (!empty($entity) && !is_null($top_level_entity)) {
          // If there is more than one it's not obvious which one is the parent,
          // so we should probably not try to edit.
          return NULL;
        }
      }
      if (!empty($top_level_entity)) {
        $this->entity = $top_level_entity;
      }
    }
    return $this->entity;
  }

  public function clearDerivedEntity() {
    $this->entity = NULL;
  }

}

==> navigation_plus-1.0.4.tar.gz: navigation_plus/src/NavigationPlusUi.php <==
<?php

declare(strict_types=1);

namespace Drupal\navigation_plus;

use Drupal\Core\Render\Element;
use Drupal\Core\Routing\AdminContext;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Routing\RouteMatchInterface;
use Drupal\Core\Session\AccountProxyInterface;
use Drupal\Core\Extension\ExtensionPathResolver;
use Symfony\Component\HttpFoundation\RequestStack;
use Drupal\Core\Entity\EntityDisplayRepositoryInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\layout_builder\OverridesSectionStorageInterface;
use Symfony\Contracts\EventDispatcher\EventDispatcherInterface;

/**
 * Navigation + UI.
 */
final class NavigationPlusUi {

  use StringTranslationTrait;

  /**
   * @var \Drupal\Core\Entity\EntityInterface|NULL
   *   The entity derived from the route.
   *   @see deriveEntityFromRoute.
   */
  private ?EntityInterface $entity = NULL;

  public function __construct(
    private readonly RequestStack $requestStack,
    private readonly RouteMatchInterface $routeMatch,
    private readonly AdminContext $routerAdminContext,
    private readonly ModePluginManager $modeManager,
    private readonly AccountProxyInterface $currentUser,
    private readonly EventDispatcherInterface  $eventDispatcher,
    private readonly ExtensionPathResolver $extensionPathResolver,
    private readonly EntityDisplayRepositoryInterface $entityDisplayRepository,
  ) {}

  /**
   * Add toolbars.
   *
   * Add mode toolbars to the navigation module's left sidebar.
   *
   * @param array $variables
   *   The navigation sidebar render array.
   *
   * @throws \Drupal\Component\Plugin\Exception\PluginException
   */
  public function buildToolbars(&$variables) {
    $mode_state = $this->getMode();
    // Hide the standard navigation items if any modes are enabled.
    if ($mode_state !== 'none') {
      foreach ($variables['content']['content'] as &$navigation_item) {
        $navigation_item['#attributes']['class'][] = 'navigation-plus-hidden';
      }
    }
    foreach ($this->modeManager->getModePlugins() as $mode_id => $mode) {
      $variables['#cache']['contexts'][] = 'user.permissions';
      $variables['#cache']['contexts'][] = "cookies:navigationMode";

      // Add a button to toggle into the mode.
      $variables['#attached']['library'][] = 'navigation_plus/navigation_plus';
      $children = Element::children($variables['content']['footer']);
      $last_key = end($children);
      $variables['content']['footer'][$last_key]['content']["navigation_plus_toggle_{$mode_id}_mode"] = [
        '#type' => 'inline_template',
        '#template' => "<style>#toggle-{{mode_id}}-mode::before { mask-image: url({{icon_path}}); }</style><a id='toggle-{{mode_id}}-mode' data-mode='{{mode_id}}' data-drupal-tooltip='{{mode}}' data-drupal-tooltip-class='admin-toolbar__tooltip' class='navigation-plus-mode-button toolbar-button toolbar-button--collapsible{{toolbar_state}}' data-index-text='0' data-icon-text='{{icon_text}}' href='javascript:void(0)'><span class='toolbar-button__label'>{{label}}</span></a>",
        '#context' => [
          'mode_id' => $mode_id,
          'label' => $mode->label(),
          'icon_text' => $mode->label(),
          'toolbar_state' => $mode_state === $mode_id ? ' active' : '',
          'mode' => t('@label mode', ['@label' => $mode->label()]),
          'icon_path' => $mode->getIconPath(),
        ],
      ];

      // Add a toolbar for this mode.
      $toolbar_classes = ['toolbar-block'];
      if ($mode_state !== $mode_id) {
        $toolbar_classes[] = 'navigation-plus-hidden';
      }
      $variables['content']['content']["navigation_plus_$mode_id"] = [
        '#type' => 'container',
        '#attributes' => [
          'id' => "navigation-plus-$mode_id",
          'class' => $toolbar_classes,
        ],
        // Match the navigation module markup.
        'admin_toolbar_item' => [
          '#type' => 'container',
          '#attributes' => [
            'class' => ['admin-toolbar__item'],
          ],
          'title' => [
            '#type' => 'html_tag',
            '#tag' => 'h4',
            '#value' => t('Editing Toolbar'),
            '#attributes' => [
              'class' => ['visually-hidden', 'focusable'],
            ],
          ],
          'tools' => [
            '#theme' => 'item_list',
            '#attributes' => [
              'class' => ['toolbar-block__list'],
            ],
            '#items' => $mode->buildToolbarItems($variables, $mode),
          ],
        ],
        '#weight' => 99,
      ];
    }
  }

  /**
   * Build page top.
   *
   * Let modes add top and side bars.
   *
   * @param array $page_top
   *   The page_top render array.
   *
   * @return void
   *
   * @throws \Drupal\Component\Plugin\Exception\PluginException
   */
  public function buildPageTop(array &$page_top) {
    foreach ($this->modeManager->getModePlugins() as $mode_id => $mode) {
      $mode->buildBars($page_top, $mode);
    }
  }

  /**
   * Preprocess top bar.
   */
  public function preprocessTopBar(&$variables) {
    $variables['#cache']['contexts'][] = 'cookies:navigationMode';
    // Hide the navigation top bar when in a mode.
    if ($this->getMode() !== 'none') {
      $variables['attributes']['class'][] = 'navigation-plus-hidden';
    }
  }


  /**
   * Get mode.
   *
   * Many UI state items are stored in JS's Local and Session storage. Edit mode
   * is stored as a cookie so that the server can conditionally render the page
   * elements with attributes used for the editing UI. Rendering this server side
   * prevents a flashing that would occur if we waited till the JS was loaded
   * to enable the editing UI.
   *
   * @param string $cookie_name
   *   The name of the cookie.
   *
   * @return string
   *   Whether edit mode is enabled (Whether the toolbar is open or closed).
   */
  public function getMode(string $cookie_name = 'navigationMode'): string {
    // Cookies are for pages like /node/10
    return $this->requestStack->getCurrentRequest()->cookies->get($cookie_name) ??
      // Query parameters are for js requests like /lb-plus/place-block/overrides/node.16
      $this->requestStack->getCurrentRequest()->get($cookie_name) ??
      'none';
  }

  /**
   * Is valid view mode.
   *
   * @return bool
   *   Whether the view_mode is a valid view mode.
   */
  public function isValidViewMode(EntityInterface $entity, string $view_mode): bool {
    $valid_view_modes = $this->entityDisplayRepository->getViewModes($entity->getEntityTypeId());
    $valid_view_modes['default'] = TRUE;
    return !empty($valid_view_modes[$view_mode]);
  }

  /**
   * Derive entity from route.
   *
   * @return \Drupal\Core\Entity\EntityInterface|null
   *   Returns the entity for this route or NULL if there isn't one or there is
   *   more than one.
   */
  public function deriveEntityFromRoute() {
    if (empty($this->entity)) {
      $parameters = $this->routeMatch->getParameters()->all();
      $top_level_entity = NULL;
      foreach ($parameters as $parameter) {
        $entity = NULL;
        if ($parameter instanceof EntityInterface) {
          $entity = $parameter;
        }
        if ($parameter instanceof OverridesSectionStorageInterface) {
          $entity = $parameter->getContextValue('entity');
        }
        if (!empty($entity) && is_null($top_level_entity)) {
          $top_level_entity = $entity;
        } elseif (!empty($entity) && !is_null($top_level_entity)) {
          // If there is more than one it's not obvious which one is the parent,
          // so we should probably not try to edit.
          return NULL;
        }
      }
      if (!empty($top_level_entity)) {
        $this->entity = $top_level_entity;
      }
    }
    return $this->entity;
  }

  public function clearDerivedEntity() {
    $this->entity = NULL;
  }

}
